package izayoi.cache.impl.selene;

import izayoi.Configures;
import izayoi.Constants;
import izayoi.Izayoi;
import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.CacheFactory;
import izayoi.cache.impl.ThumbNailedWrapper;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * replace of net.sourceforge.shingetsu.cache.CacheManager
 * @author es
 */
public final class SeleneWrapperFactory extends CacheFactory {

    class Filelist implements FilenameFilter {
        String type;

        public Filelist(String type) {
            this.type = ((type == null) ? "" : type) + "_";
        }
        public boolean accept(File arg0, String arg1) {
            return arg1.startsWith(type);
        }
    }

    public static final String CONFIG_SUFFIX = "SeleneWrapper.cache.suffix";
    private String _suffix;
    private File path;

    /* ( Javadoc)
     * @see izayoi.cache.CacheFactory#getEntris(java.lang.String)
     */
    public CacheEntry[] getEntris(String type) throws IOException {
        File[] files = path.listFiles(new Filelist(type));
        List l = new ArrayList();
        for (int i = 0; i < files.length; i++) {
            CacheEntry ent = null;
            try {
                ent = new SeleneBindEntry(this, files[i]);
                if (isMakeThumbNail() && type.equals(Constants.TYPE_THREAD)) {
                    ent = new ThumbNailedWrapper(ent, thumbNailPath, thumbNailFormat);
                }
            } catch (IOException e) {
                System.err.println(e);
                continue;
            }
            if (ent != null) {
                l.add(ent);
            }
        }

        CacheEntry[] ret = new CacheEntry[l.size()];
        ret = (CacheEntry[]) l.toArray(ret);
        return ret;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheFactory#getEntry(java.lang.String, java.lang.String, boolean)
     */
    public CacheEntry getEntry(String type, String basename, boolean isCreate) throws IOException {
        if (type == null || basename == null || type.equals("") || basename.equals("")) {
            throw new IllegalArgumentException("bad entry name");
        }
        String key = type + "_" + Util.parseHexString(basename) + _suffix;
        File target = new File(path, key);
        if (!target.exists()) {
            if (isCreate) {
                target.createNewFile();
            } else {
                return null;
            }
        }

        CacheEntry ent = new SeleneBindEntry(this, target, type, basename);
        if (isMakeThumbNail() && type.equals(Constants.TYPE_THREAD)) {
            ent = new ThumbNailedWrapper(ent, thumbNailPath, thumbNailFormat);
        }
        return ent;
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheFactory#init(izayoi.Configures)
     */
    public void init(Configures c) {
        path = new File(c.getParameter(Izayoi.CONFIG_CACHE_STOREPATH, "./"));
        _suffix = c.getParameter(CONFIG_SUFFIX, "");
    }

    /* ( Javadoc)
     * @see izayoi.cache.CacheFactory#isExist(java.lang.String, java.lang.String)
     */
    public boolean isExist(String type, String basename) throws IOException {
        return (this.getEntry(type, basename, false) != null);
    }

    /* ( Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getClass().getName();
    }

}
