/**
 * ThumbNailMaker.java
 * Created on 2005/06/13
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.misc;

import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;

/**
 * kڍ쐬
 * @author es
 */
public class ThumbNailMaker {

    private class ThreadHolder {
        private Thread thread;
        ThreadHolder(Thread t) {
            thread = t;
        }
        synchronized void clear() {
            thread = null;
        }
        synchronized Thread get() {
            return thread;
        }
    }

    private String format;
    private ThreadHolder _thread;
    private File filename;
    private BufferedImage image;

	//@TODO ʂɌŒł˂?
    private int limit_w = 80;
    private int limit_h = 60;

    private ThumbNailMaker() {
        final Runnable runner = new Runnable() {
            public void run() {
                try {
                    makeit();
                } catch (Exception e) {
                    throw new RuntimeException(e);
                } finally {
                    _thread.clear();
                }
            }
        };
        _thread = new ThreadHolder(new Thread(runner));
    }

    public ThumbNailMaker(File storePath, BufferedImage image) {
        this(storePath, image, "png");
    }

    public ThumbNailMaker(File storePath, BufferedImage image, String format) {
        this();
        this.image = image;
        this.filename = storePath;
        this.format = format;
    }

    public final void execute() {
        Thread t = _thread.get();
        if (t != null) {
            t.start();
        }
    }

    public final void interrupt() {
        Thread t = _thread.get();
        if (t != null) {
            t.interrupt();
        }
        _thread.clear();
    }

    private void makeit() throws Exception {
        if (image == null || filename == null) {
            return;
        }

        int w = image.getWidth();
        int h = image.getHeight();
        double d;
        if (h > limit_h) {
            d = (double) limit_h / h;
        } else if (w > limit_w) {
            d = (double) limit_w / w;
        } else {
            d = 1.0d;
        }

        BufferedImage dst = new BufferedImage((int) (w * d), (int) (h * d), image.getType());
        AffineTransformOp op = new AffineTransformOp(AffineTransform.getScaleInstance(d, d), null);
        op.filter(image, dst);

		//@TODO o̓tH[}bggif̏ꍇɌF&IndexColoredModelŏĂ
		//(ʓ|pngŒł)

        if(!filename.getParentFile().exists()) {
            filename.getParentFile().mkdir();
        }
        ImageIO.write(dst, format, filename);
    }

}
