/**
 * GenericNode.java
 * Created on 2005/05/20
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.node;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;

import edu.emory.mathcs.backport.java.util.concurrent.LinkedBlockingQueue;

/**
 * ėpm[h
 * @author es
 */
public class GenericNode implements AsynchronousNode {

    protected Collection listeneres;
    protected int port = 80;
    protected URL target;
    protected String userAgent;

    /**
     * @param url m[hurl
     */
    public GenericNode(URL url) {
        listeneres = new LinkedBlockingQueue();
        target = url;
        init();
    }

    /**
     * @param url m[hurl
     * @param port m[h̃|[gԍ
     */
    public GenericNode(URL url, int port) {
        this(url);
        this.port = port;
    }

    private TalkingEvent _bye(boolean isAsync) {
        TalkingQuery[] q =
            new TalkingQuery[] { parseQuery("ping", null), parseQuery("bye", "@backward[1]@:" + port + "+server.cgi")};
        return batch(q, isAsync);
    }

    private TalkingEvent _get(String filename, String timeArgument, boolean isAsync) {
        TalkingQuery[] q =
            new TalkingQuery[] {
                parseQuery("ping", null),
                parseQuery("have", filename),
                parseQuery("get", filename + "/" + timeArgument)};
        return batch(q, isAsync);
    }

    private TalkingEvent _have(String filename, boolean isAsync) {
        TalkingQuery[] q = new TalkingQuery[] { parseQuery("ping", null), parseQuery("have", filename)};
        return batch(q, isAsync);
    }

    private TalkingEvent _head(String filename, String timeArgument, boolean isAsync) {
        TalkingQuery[] q =
            new TalkingQuery[] {
                parseQuery("ping", null),
                parseQuery("have", filename),
                parseQuery("head", filename + "/" + timeArgument)};
        return batch(q, isAsync);

    }

    private TalkingEvent _join(boolean isAsync) {
        TalkingQuery[] q =
            new TalkingQuery[] { parseQuery("ping", null), parseQuery("join", "@backward[1]@:" + port + "+server.cgi")};
        return batch(q, isAsync);
    }

    private TalkingEvent _node(boolean isAsync) {
        TalkingQuery[] q = new TalkingQuery[] { parseQuery("ping", null), parseQuery("node", null)};
        return batch(q, isAsync);
    }

    private TalkingEvent _update(String filename, String timeArgument, String md5, String nodeStr, boolean isAsync) {
        TalkingQuery[] q =
            new TalkingQuery[] {
                parseQuery("ping", null),
                parseQuery("update", filename + "/" + timeArgument + "/" + md5 + "/" + nodeStr)};
        return batch(q, isAsync);
    }

    /* ( Javadoc)
      * @see izayoi.node.AsynchronousNode#addNodeListener(izayoi.node.AsynchronousNodeListener)
      */
    public void addNodeListener(AsynchronousNodeListener listener) {
        if (!listeneres.contains(listener)) {
            listeneres.add(listener);
        }
    }

    /**
     * #bye() ̔񓯊^
     * @see bye()
     */
    public void asyncBye() {
        _bye(true);
    }

    /**
     * #get(java.lang.String, java.lang.String) ̔񓯊^
     * @param filename t@C
     * @param timeArgument w
     * @see GenericNode#get(java.lang.String, java.lang.String)
     */
    public void asyncGet(String filename, String timeArgument) {
        _get(filename, timeArgument, true);
    }

    /**
     * #have(java.lang.String) ̔񓯊^
     * @param filename t@C
     * @see #have(java.lang.String)
     */
    public void asyncHave(String filename) {
        _have(filename, true);
    }

    /**
     * #head(java.lang.String, java.lang.String) ̔񓯊^
     * @param filename t@C
     * @param timeArgument w
     * @see #head(java.lang.String, java.lang.String)
     */
    public void asyncHead(String filename, String timeArgument) {
        _head(filename, timeArgument, true);
    }

    /**
     * #join() ̔񓯊^
     * @see #join()
     */
    public void asyncJoin() {
        _join(true);
    }

    /**
     * #node() ̔񓯊^
     * @see #node()
     */
    public void asyncNode() {
        _node(true);
    }

    /**
     * #ping() ̔񓯊^
     * @see #ping()
     */
    public void asyncPing() {
        asyncTalk("ping", null);
    }

    /**
     * #root() ̔񓯊^
     * @see #root()
     */
    public void asyncRoot() {
        asyncTalk(null, null);
    }

    /**
     * R}hAƂɓ'/'Ȃ
     * @param command R}h(w/x𔭍sꍇnull)
     * @param argument R}hȍ~̈(ȂȂnull)
     * @see izayoi.node.AsynchronousNode#asyncTalk(java.lang.String, java.lang.String)
     */
    public void asyncTalk(String command, String argument) {
        new AsynchronousNodeTalker(this, parseQuery(command, argument)).talk();
    }

    /**
     * #update() ̔񓯊^
     * @param filename
     * @param timeArgument
     * @param md5
     * @param nodeStr
     * @see #update()
     */
    public void asyncUpdate(String filename, String timeArgument, String md5, String nodeStr) {
        _update(filename, md5, timeArgument, nodeStr, true);
    }

    protected TalkingEvent batch(TalkingQuery[] q, boolean isAsync) {
        if (isAsync) {
            new AsynchronousNodeTalker(this, q).talk();
            return null;
        } else {
            return new TalkingQueryChain(q).invoke();
        }
    }

    /**
     * w/byex𔭍s
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#bye()
     */
    public void bye() throws ProtocolException, IOException {
        _bye(false);
    }

    /* ( Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object arg0) {
        return this.getServantURL().equals(arg0);
    }

    /* ( Javadoc)
     * @see izayoi.node.AsynchronousNode#fire(izayoi.node.NodeEvent)
     */
    public void fire(TalkingEvent e) {
        for (Iterator i = listeneres.iterator(); i.hasNext();) {
            ((AsynchronousNodeListener) i.next()).commFinished(e);
        }
    }

    /**
     * w/getx𔭍s
     * @param filename t@C
     * @param timeArgument w
     * @return Ɉv郌R[h
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#get(java.lang.String, java.lang.String)
     */
    public String[] get(String filename, String timeArgument) throws ProtocolException, IOException {
        TalkingEvent e = parseEvent(_get(filename, timeArgument, false));
        if (e.getNegateDetail() != null) {
            throw new ProtocolException(e.getNegateDetail().getMessage());
        }
        return e.getResponse();
    }

    /* ( Javadoc)
     * @see izayoi.node.AsynchronousNode#getNodeListeners()
     */
    public AsynchronousNodeListener[] getNodeListeners() {
        return (AsynchronousNodeListener[]) listeneres.toArray(new AsynchronousNodeListener[listeneres.size()]);
    }

    /* ( Javadoc)
     * @see izayoi.node.AbstractNode#getURL()
     */
    public URL getServantURL() {
        return target;
    }

    /**
     * w/havex𔭍s
     * @param filename t@C
     * @return t@CL
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     */
    public boolean have(String filename) throws ProtocolException, IOException {
        boolean result = (parseEvent(_have(filename, false)).getNegateDetail() == null);
        return result;
    }

    /**
     * w/headx𔭍s
     * @param filename t@C
     * @param timeArgument w
     * @return Ɉv郌R[h
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#head(java.lang.String, java.lang.String)
     */
    public String[] head(String filename, String timeArgument) throws ProtocolException, IOException {
        TalkingEvent e = parseEvent(_head(filename, timeArgument, false));
        if (e.getNegateDetail() != null) {
            ProtocolException ex = new ProtocolException(e.getNegateDetail().getMessage());
            ex.initCause(e.getNegateDetail());
            throw ex;
        }
        return e.getResponse();
    }

    /**
     * 
     */
    synchronized protected void init() {
        userAgent = "shinGETsu/0.5";
    }

    /**
     * w/joinx𔭍s
     * @return m[hURLA܂͎g
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#join()
     */
    public URL join() throws ProtocolException, IOException {
        String[] ret = parseEvent(_join(false)).getResponse();
        if (ret.length > 1) {
            return new URL("http://" + ret[1].trim());
        } else {
            return getServantURL();
        }
    }

    /**
     * w/nodex𔭍s
     * @return m[hURL
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#node()
     */
    public URL node() throws ProtocolException, IOException {
        String[] ret = parseEvent(_node(false)).getResponse();
        return new URL("http://" + ret[0].trim());
    }

    /**
     * @param event
     * @return
     */
    protected TalkingEvent parseEvent(TalkingEvent event) throws ProtocolException, IOException {
        NegateResponse neg = event.getNegateDetail();
        if (neg == null) {
            return event;
        }

        Throwable cause = neg.getCause();
        if (cause == null) {
            return event;
        }

        if (cause instanceof ProtocolException) {
            throw (ProtocolException) cause;
        } else if (cause instanceof IOException) {
            throw (IOException) cause;
        } else {
            IOException ex = new IOException(cause.getMessage());
            ex.initCause(cause);
            throw ex;
        }
    }

    protected TalkingQuery parseQuery(String command, String argument) {
        StringBuffer sb = new StringBuffer(target.toString());
        sb.append('/');
        if (command != null) {
            sb.append(command);
            if (argument != null) {
                sb.append('/');
                sb.append(argument);
            }
        }
        try {
            return new TalkingQuery(new URL(sb.toString()), userAgent);
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException(e.getMessage());
        }
    }

    /**
     * w/pingx𔭍s
     * @return 悪LȂ^
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#ping()
     */
    public boolean ping() throws ProtocolException, IOException {
        try {
            talk("ping", null);
        } catch (ProtocolException e) {
            return false;
        } catch (IOException e) {
            throw e;
        }
        return true;
    }

    /* ( Javadoc)
     * @see izayoi.node.AsynchronousNode#removeNodeListener(izayoi.node.AsynchronousNodeListener)
     */
    public void removeNodeListener(AsynchronousNodeListener listener) {
        if (listeneres.contains(listener)) {
            listeneres.remove(listener);
        }
    }

    /**
     * w/x𔭍s
     * @return 扞bZ[W
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#root()
     */
    public String[] root() throws ProtocolException, IOException {
        return talk(null, null);
    }

    /**
     * R}hAƂɓ'/'Ȃ
     * @param command R}h(w/x𔭍sꍇnull)
     * @param argument R}hȍ~̈(ȂȂnull)
     * @see izayoi.node.AbstractNode#talk(java.lang.String, java.lang.String)
     */
    public String[] talk(String command, String argument) throws IOException, ProtocolException {
        TalkingQuery q = parseQuery(command, argument);
        switch (q.talk()) {
            case TalkingQuery.RESULT_ACK :
                return q.getResponse();
            case TalkingQuery.RESULT_NEG :
                NegateResponse neg = new NegateResponse(q);
                ProtocolException ex = new ProtocolException(neg.getMessage());
                ex.initCause(neg);
                throw ex;
            default :
                throw new ProtocolException("X|XbZ[Wsł");
        }
    }

    /* ( Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getServantURL().toString();
    }

    /**
     * w/updatex𔭍s
     * @param filename
     * @param timeArgument
     * @param md5
     * @param nodeStr
     * @throws ProtocolException X|Xsȏꍇ
     * @throws IOException ʐMɓo̓G[ꍇ
     * @see net.sourceforge.shingetsu.node.Node#update(java.lang.String, java.lang.String, java.lang.String, net.sourceforge.shingetsu.node.Node)
     */
    public void update(String filename, String timeArgument, String md5, String nodeStr)
        throws IOException, ProtocolException {
        parseEvent(_update(filename, md5, timeArgument, nodeStr, true));
    }

}
