/**
 * TalkingQueryChain.java
 * Created on 2005/05/20
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.node;

import java.net.ProtocolException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * TalkingQuerÿꊇs`
 * @author es
 */
public class TalkingQueryChain {

    private List query;
    private int pos = 0;

    //Qƃp^[
    private static final Pattern backward = Pattern.compile("@backward\\[(\\d)\\]@");

    /**
     * RXgN^
     * @param query ʐMNG
     */
    public TalkingQueryChain(TalkingQuery query) {
        this.query = new ArrayList();
        this.query.add(query);
    }

    /**
     * RXgN^
     * @param queries ʐMNG
     */
    public TalkingQueryChain(TalkingQuery[] queries) {
        this.query = new ArrayList();
        this.query.addAll(Arrays.asList(queries));
    }

    /**
     * ꊇs
     * @return s
     */
    public TalkingEvent invoke() {
        return invoke(null);
    }

    /**
     * ꊇs
     * @param node Ăь̃m[h
     * @return s
     */
    public TalkingEvent invoke(AsynchronousNode node) {
        TalkingQuery q = null;
        try {
            while (hasNext()) {
                q = traverse(get());
                switch (q.talk()) {
                    case TalkingQuery.RESULT_ACK :
                        next();
                        break;
                    case TalkingQuery.RESULT_NEG :
                        throw new NegateResponse(q);
                    case TalkingQuery.RESULT_ERR :
                        throw new ProtocolException("X|XbZ[Wsł");
                }
            }
        } catch (Throwable e) {
            NegateResponse ex;
            if (!(e instanceof NegateResponse)) {
                ex = new NegateResponse(q, e);
            } else {
                ex = (NegateResponse) e;
            }
            q = getLast();
            return new TalkingEvent(node, q.getCommand(), q.getArgument(), ex);
        }

        return new TalkingEvent(node, q.getCommand(), q.getArgument(), q.getResponse());
    }

    /*
     * Q
     * @param query
     * @return
     */
    private TalkingQuery traverse(TalkingQuery query) throws Exception {
        Matcher m = backward.matcher(query.getFile());
        if (!m.find()) {
            return query;
        }

        String mod = prev().getResponse()[Integer.parseInt(m.group(1))];
        StringBuffer sb = new StringBuffer();
        m.appendReplacement(sb, mod);
        m.appendTail(sb);
        query.setFile(sb.toString());
        return query;
    }

    /*
     * @return
     */
    private TalkingQuery getLast() {
        return (TalkingQuery) query.get(query.size() - 1);
    }

    /*
     * 
     */
    private void next() {
        pos++;
    }

    /*
     * @return
     */
    private boolean hasNext() {
        return pos < query.size();
    }

    /*
     * @return
     */
    private TalkingQuery get() {
        return (TalkingQuery) query.get(pos);
    }

    /*
     * @return
     */
    private TalkingQuery prev() {
        return (TalkingQuery) query.get(pos - 1);
    }

}
