package izayoi.servlet;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.ListRecord;
import izayoi.cache.Record;
import izayoi.exception.InvalidParameter;
import izayoi.exception.NotImplementedException;
import izayoi.view.DataContainer;
import izayoi.view.TemplateContext;
import izayoi.view.ViewList;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Replace of ListCGI.java
 * @author es
 */
public class ListServlet extends BasementServlet {

    /* ( Javadoc)
     * @see izayoi.servlet.BasementServlet#execute(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String[] path = parseURL(request.getRequestURI());
        if (path == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        String command = request.getParameter(PARAM_COMMAND);
        String name = (path[0] == null) ? "menu" : Util.urlDecode(path[0]);

        if (command == null) {
            //Xg\
            getList(request, response, name);
        } else if (request.getMethod().equalsIgnoreCase("post") && command.equals(ACTION_CREATE)) {
            //Xgǉ
            addList(request, response, name);
        } else {
            throw new NotImplementedException(command);
        }
    }

    /**
     * Xg\
     * @param request
     * @param response
     * @param basename Ώۂ̃x[Xl[
     * @throws ServletException
     * @throws IOException
     */
    private void getList(HttpServletRequest request, HttpServletResponse response, String basename)
        throws ServletException, IOException {
        CacheEntry entry = cache.getEntry(Constants.TYPE_LIST, basename);
        String filename = (entry == null) ? basename : entry.getBaseName();

		//\[g擾
		Map reqMap = request.getParameterMap();
		String sort = (String) request.getAttribute(PARAM_SORT_LIST);
		if(reqMap.containsKey(PARAM_SORT)) {
			sort = request.getParameter(PARAM_SORT);
		}
		String order = (String) request.getAttribute(PARAM_ORDER_LIST);
		if(reqMap.containsKey(PARAM_ORDER)) {
			order = request.getParameter(PARAM_ORDER);
		}
        order = (order == null) ? "" : order;

		//R[h擾
        Record[] record = entry == null ? null : entry.getRecords();
        DataContainer[] data = ViewList.parseContent(record, sort, !order.equals(Constants.SORT_DESC));

        TemplateContext ctx = new TemplateContext("list.txt", request, response);
        ctx.setRecord(data);
        ctx.setName(filename);
        ctx.setSize(entry != null ? entry.getSize() : 0);
        ctx.finish();
    }

    /**
     * Xgǉ
     * @param request
     * @param response
     * @param basename Ώۂ̃x[Xl[
     * @throws ServletException
     * @throws IOException
     */
    private void addList(HttpServletRequest request, HttpServletResponse response, String basename)
        throws ServletException, IOException {
        String name = request.getParameter(PARAM_NAME);
        String type = request.getParameter(PARAM_TYPE);
        boolean isNotice = (request.getParameter(PARAM_DOPOST) != null);

        if (type == null
            || (!type.equals(Constants.TYPE_LIST) && !type.equals(Constants.TYPE_NOTE) && !type.equals(Constants.TYPE_THREAD))) {
            if (type == null) {
                type = "w肪܂";
            }
            throw new InvalidParameter("", type);
        }

        name = (name == null) ? "" : name.trim();
        name = Util.escape(name);

        Record record;
        try {
            record = new ListRecord(System.currentTimeMillis() / 1000, type, name);
        } catch (Exception e) {
            throw new InvalidParameter("O", e);
        }
        CacheEntry entry = cache.getEntry(Constants.TYPE_LIST, basename, true);
        if (entry.getRecord(record.getID()) != null) {
            throw new ServletException("ɑ݂܂");
        }

        entry.append(record);
        if (!entry.isAutoCommit()) {
            entry.commit();
        }
        if (isNotice) {
            try {
                entry.notineMe(record);
            } catch (IOException e1) { //Ȃ
            }
        }

        response.sendRedirect(request.getRequestURI());
    }

}
