package net.prixdami.alexis.multipart;

import java.io.File;
import java.io.IOException;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author xsugawrk
 */
public class MultipartHandlingFilter implements Filter {

    public static final String PARAM_STOREPATH = "storepath";
    public static final String PARAM_LIMIT = "limitsize";
    public static final String PARAM_ENCODE = "default-encoding";
    private static final String METHOD = "POST";

    private String storePath;
    private long limitSize;

    private ServletContext ctx; //ServletContext of this
    private String thisName; //this filter name
    private String encoding; //header-encoding

    /**
     * 
     */
    public MultipartHandlingFilter() {
    }

    /* ( Javadoc)
     * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
     */
    public void init(FilterConfig conf) throws ServletException {
        thisName = conf.getFilterName();
        ctx = conf.getServletContext();

        //sets max request size parameter (default value is non-limit)
        String s = conf.getInitParameter(PARAM_LIMIT);
        limitSize = Long.parseLong((s == null) ? "-1" : s);

        //sets storePath parameter (default value is app's current)
        s = conf.getInitParameter(PARAM_STOREPATH);
        String root = ctx.getRealPath("./");
        storePath = new File(root, (s == null) ? root : s).getAbsolutePath();

        //sets default header encodeing parameter (default value is JVM's default)
        encoding = conf.getInitParameter(PARAM_ENCODE);
    }

    /* ( Javadoc)
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
     */
    public void doFilter(ServletRequest arg0, ServletResponse arg1, FilterChain arg2)
        throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) arg0;
        HttpServletResponse response = (HttpServletResponse) arg1;

        //		response.setLocale(request.getLocale());
        //		request.setCharacterEncoding(response.getCharacterEncoding());

        if (METHOD.equals(request.getMethod()) && MultipartStripper.isMultipartContent(request)) {
            try {
                request = parse(request);
            } catch (LimitSizeOverFlow e) {
                // limit-size overflow (HTTP 413)
                response.sendError(HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE);
                return;
            }
        }
        arg2.doFilter(request, response);
    }

    /* ( Javadoc)
     * @see javax.servlet.Filter#destroy()
     */
    public void destroy() {
    }

    /**
     * parse to multipart-request
     * @param request
     * @return modified-HttpServletRequest
     * @throws IOException
     */
    private HttpServletRequest parse(HttpServletRequest request) throws IOException {
        MultipartStripper stripper = new MultipartStripper(encoding, storePath);
        Map items = stripper.strip(request, limitSize);

        SneakMultipartRequest req = new SneakMultipartRequest(request);
        List storefiles = new ArrayList();
        for (Iterator i = items.keySet().iterator(); i.hasNext();) {
            String key = (String) i.next();
            Object item = items.get(key);

            if (item instanceof MultipartFile) {
                storefiles.add((MultipartFile) item);
                continue;
            }
            if (item instanceof List) {
                List l = (List) item;
                String[] str = new String[l.size()];
                l.toArray(str);
                req.setParameter(key, str);
                continue;
            }

        }

        if (storefiles.size() != 0) {
            MultipartFile[] store = new MultipartFile[storefiles.size()];
            storefiles.toArray(store);
            req.setAttribute(MultipartFile.MULTIPART_ITEMS, store);
        }

        return req;
    }

    /**
     * logging
     * @param String log-message
     */
    public void log(String arg) {
        ctx.log(thisName + ":" + arg);
    }

    /**
     * logging
     * @param String log-message
     * @param Exception exception
     */
    public void log(String arg, Exception ex) {
        ctx.log(thisName + ":" + arg, ex);
    }

}
