/*
 * HttpRequestImpl.java
 *
 * Created on 2004/08/20, 12:14
 */

package net.sourceforge.shingetsu.server;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.net.URL;
import java.net.InetAddress;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Map;
import java.util.Enumeration;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sourceforge.shingetsu.util.Config;
import net.sourceforge.shingetsu.server.HttpServer;
import net.sourceforge.shingetsu.server.HttpRequest;
import net.sourceforge.shingetsu.server.HttpRequestProcessor;

import java.lang.IllegalArgumentException;
import java.lang.NumberFormatException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.io.UnsupportedEncodingException;

import net.sourceforge.shingetsu.server.HttpProtocolException;

/**
 * HttpRequestC^[tFCX̎NXłB
 * @author stokesia
 */
public class HttpRequestImpl implements HttpRequest {
    
    private InetAddress inetAddress = null;
    
    private Config config = Config.getInstance();
    
    private String method = null;
    private String originalRequestURI= null;
    private String httpVersion = null;
    
    private URL url = null;
    
    private Hashtable messageHeaderTable = null;
    private Hashtable queryParameterTable = null;
    
    private boolean multipart = false;

    /**
     * HttpRequestImpl̃CX^X𐶐܂B
     */    
    protected HttpRequestImpl() {
    }

    /**
     * URLNX̃IuWFNgZbg܂B
     * @param url URLNX̃IuWFNg
     */    
    protected void setURL(URL url) {
        this.url = url;
    }
    
    /**
     * GETAPOST̃NGXgNG[̃e[uZbg܂B
     * @param queryParameterTable NG[̃e[u
     */    
    protected void setQueryParameterTable(Hashtable queryParameterTable) {
        this.queryParameterTable = queryParameterTable;
    }
    
    /**
     * ̃NGXg}`p[gǂZbg܂B
     * @param multipart }`p[g̃NGXgȂtrue
     */    
    protected void setMultipatRequest(boolean multipart) {
        this.multipart = multipart;
    }
    
    public boolean isMultipartRequest() {
        return multipart;
    }
    
    /**
     * bZ[Wwb_[̃e[uZbg܂B
     * @param headerTable bZ[Wwb_[̃e[u
     */    
    protected void setMessageHeaderTable(Hashtable headerTable) {
        messageHeaderTable = headerTable;
    }
    
    /**
     * NGXg̑MInetAddressNX̃IuWFNgZbg܂B
     * @param address NGXg̑MInetAddressNX̃IuWFNg
     */    
    protected void setInetAddress(InetAddress address) {
        this.inetAddress = address;
    }
    
    public String getProtocolVersion() {
        return httpVersion;
    }
    
    /**
     * ̃NGXg̃vgRo[WZbg܂B
     * @param httpVersion ̃NGXgHTTP̃o[W
     */    
    protected void setProtocolVersion(String httpVersion) {
        this.httpVersion = httpVersion;
    }
    
    public String getMethod() {
        return method;
    }
    
    /**
     * ̃NGXgɗpꂽ\bhZbg܂B
     * @param method ̃NGXg̃\bh
     */    
    protected void setMethod(String method) {
        this.method = method;
    }
    
    public String getPath() {
        return url.getPath();
    }
    
    public String getQuery() {
        return url.getQuery();
    }
    
    public String getParameter(String name) {
        String value = null;
        
        if(queryParameterTable != null) {
            if((name != null) && queryParameterTable.containsKey(name)) {
                value = (String) queryParameterTable.get(name);
            }
        } else {
            if(name != null) {
                Map part = getPart(name);
                if((part != null) && part.containsKey(name)) {
                    value = (String) part.get(name);
                } else {
                    if(messageHeaderTable.containsKey(name)) {
                        value = (String) messageHeaderTable.get(name);
                    }
                }
            }
        }
        
        return value;
    }
    
    public String getHeader(String name) {
        String value = null;
        
        if(messageHeaderTable != null) {
            if((name != null) && messageHeaderTable.containsKey(name)) {
                value = (String) messageHeaderTable.get(name);
            }
        }
            
        return value;
    }
    
    public String getDateHeader(String name) {//throws IllegalArgumentException {
        return null;
    }
    
    public String getIntHeader(String name) { //throws NumberFormatException {
        return null;
    }
    
    public Enumeration getHeaderNames() {
        return messageHeaderTable.keys();
    }
    
    public Enumeration getHeaders() {
        return messageHeaderTable.elements();
    }
    
    public String getHostAddress() {
        return inetAddress.getHostAddress();
    }
    
    public String getRemoteHost() {
        return inetAddress.getHostName();
    }
    
    public String getFile() {
        return url.getFile();
    }
    
    public Map getPart(String name) {
        Map value = null;
        
        if((name != null) && messageHeaderTable.containsKey(name)) {
            Object object = messageHeaderTable.get(name);
            
            if(object instanceof Map) {
                value = (Map) object;
            }
        }
        
        return value;
    }
    
}
